import contextlib

import pytest
asyncio = pytest.importorskip("asyncio")
aiohttp = pytest.importorskip("aiohttp")

import vcr  # noqa: E402
from .aiohttp_utils import aiohttp_app, aiohttp_request  # noqa: E402


def run_in_loop(fn):
    with contextlib.closing(asyncio.new_event_loop()) as loop:
        asyncio.set_event_loop(loop)
        task = loop.create_task(fn(loop))
        return loop.run_until_complete(task)


def request(method, url, output='text', **kwargs):
    def run(loop):
        return aiohttp_request(loop, method, url, output=output, **kwargs)

    return run_in_loop(run)


def get(url, output='text', **kwargs):
    return request('GET', url, output=output, **kwargs)


def post(url, output='text', **kwargs):
    return request('POST', url, output='text', **kwargs)


@pytest.fixture(params=["https", "http"])
def scheme(request):
    '''Fixture that returns both http and https.'''
    return request.param


def test_status(tmpdir, scheme):
    url = scheme + '://httpbin.org'
    with vcr.use_cassette(str(tmpdir.join('status.yaml'))):
        response, _ = get(url)

    with vcr.use_cassette(str(tmpdir.join('status.yaml'))) as cassette:
        cassette_response, _ = get(url)
        assert cassette_response.status == response.status
        assert cassette.play_count == 1


def test_headers(tmpdir, scheme):
    url = scheme + '://httpbin.org'
    with vcr.use_cassette(str(tmpdir.join('headers.yaml'))):
        response, _ = get(url)

    with vcr.use_cassette(str(tmpdir.join('headers.yaml'))) as cassette:
        cassette_response, _ = get(url)
        assert cassette_response.headers == response.headers
        assert cassette.play_count == 1


def test_text(tmpdir, scheme):
    url = scheme + '://httpbin.org'
    with vcr.use_cassette(str(tmpdir.join('text.yaml'))):
        _, response_text = get(url)

    with vcr.use_cassette(str(tmpdir.join('text.yaml'))) as cassette:
        _, cassette_response_text = get(url)
        assert cassette_response_text == response_text
        assert cassette.play_count == 1


def test_json(tmpdir, scheme):
    url = scheme + '://httpbin.org/get'
    headers = {'Content-Type': 'application/json'}

    with vcr.use_cassette(str(tmpdir.join('json.yaml'))):
        _, response_json = get(url, output='json', headers=headers)

    with vcr.use_cassette(str(tmpdir.join('json.yaml'))) as cassette:
        _, cassette_response_json = get(url, output='json', headers=headers)
        assert cassette_response_json == response_json
        assert cassette.play_count == 1


def test_binary(tmpdir, scheme):
    url = scheme + '://httpbin.org/image/png'
    with vcr.use_cassette(str(tmpdir.join('binary.yaml'))):
        _, response_binary = get(url, output='raw')

    with vcr.use_cassette(str(tmpdir.join('binary.yaml'))) as cassette:
        _, cassette_response_binary = get(url, output='raw')
        assert cassette_response_binary == response_binary
        assert cassette.play_count == 1


def test_post(tmpdir, scheme):
    data = {'key1': 'value1', 'key2': 'value2'}
    url = scheme + '://httpbin.org/post'
    with vcr.use_cassette(str(tmpdir.join('post.yaml'))):
        _, response_json = post(url, data=data)

    with vcr.use_cassette(str(tmpdir.join('post.yaml'))) as cassette:
        _, cassette_response_json = post(url, data=data)
        assert cassette_response_json == response_json
        assert cassette.play_count == 1


def test_params(tmpdir, scheme):
    url = scheme + '://httpbin.org/get'
    headers = {'Content-Type': 'application/json'}
    params = {'a': 1, 'b': False, 'c': 'c'}

    with vcr.use_cassette(str(tmpdir.join('get.yaml'))) as cassette:
        _, response_json = get(url, output='json', params=params, headers=headers)

    with vcr.use_cassette(str(tmpdir.join('get.yaml'))) as cassette:
        _, cassette_response_json = get(url, output='json', params=params, headers=headers)
        assert cassette_response_json == response_json
        assert cassette.play_count == 1


def test_params_same_url_distinct_params(tmpdir, scheme):
    url = scheme + '://httpbin.org/get'
    headers = {'Content-Type': 'application/json'}
    params = {'a': 1, 'b': False, 'c': 'c'}

    with vcr.use_cassette(str(tmpdir.join('get.yaml'))) as cassette:
        _, response_json = get(url, output='json', params=params, headers=headers)

    with vcr.use_cassette(str(tmpdir.join('get.yaml'))) as cassette:
        _, cassette_response_json = get(url, output='json', params=params, headers=headers)
        assert cassette_response_json == response_json
        assert cassette.play_count == 1

    other_params = {'other': 'params'}
    with vcr.use_cassette(str(tmpdir.join('get.yaml'))) as cassette:
        response, cassette_response_text = get(url, output='text', params=other_params)
        assert 'No match for the request' in cassette_response_text
        assert response.status == 599


def test_params_on_url(tmpdir, scheme):
    url = scheme + '://httpbin.org/get?a=1&b=foo'
    headers = {'Content-Type': 'application/json'}

    with vcr.use_cassette(str(tmpdir.join('get.yaml'))) as cassette:
        _, response_json = get(url, output='json', headers=headers)
        request = cassette.requests[0]
        assert request.url == url

    with vcr.use_cassette(str(tmpdir.join('get.yaml'))) as cassette:
        _, cassette_response_json = get(url, output='json', headers=headers)
        request = cassette.requests[0]
        assert request.url == url
        assert cassette_response_json == response_json
        assert cassette.play_count == 1


def test_aiohttp_test_client(aiohttp_client, tmpdir):
    loop = asyncio.get_event_loop()
    app = aiohttp_app()
    url = '/'
    client = loop.run_until_complete(aiohttp_client(app))

    with vcr.use_cassette(str(tmpdir.join('get.yaml'))):
        response = loop.run_until_complete(client.get(url))

    assert response.status == 200
    response_text = loop.run_until_complete(response.text())
    assert response_text == 'hello'

    with vcr.use_cassette(str(tmpdir.join('get.yaml'))) as cassette:
        response = loop.run_until_complete(client.get(url))

    request = cassette.requests[0]
    assert request.url == str(client.make_url(url))
    response_text = loop.run_until_complete(response.text())
    assert response_text == 'hello'
    assert cassette.play_count == 1



# -*- coding: utf-8 -*-
'''Tests for cassettes with custom persistence'''

# External imports
import os
from six.moves.urllib.request import urlopen

# Internal imports
import vcr
from vcr.persisters.filesystem import FilesystemPersister


class CustomFilesystemPersister(object):
    '''Behaves just like default FilesystemPersister but adds .test extension
       to the cassette file'''
    @staticmethod
    def load_cassette(cassette_path, serializer):
        cassette_path += '.test'
        return FilesystemPersister.load_cassette(cassette_path, serializer)

    @staticmethod
    def save_cassette(cassette_path, cassette_dict, serializer):
        cassette_path += '.test'
        FilesystemPersister.save_cassette(cassette_path, cassette_dict,
                                          serializer)


def test_save_cassette_with_custom_persister(tmpdir, httpbin):
    '''Ensure you can save a cassette using custom persister'''
    my_vcr = vcr.VCR()
    my_vcr.register_persister(CustomFilesystemPersister)

    # Check to make sure directory doesnt exist
    assert not os.path.exists(str(tmpdir.join('nonexistent')))

    # Run VCR to create dir and cassette file using new save_cassette callback
    with my_vcr.use_cassette(str(tmpdir.join('nonexistent', 'cassette.yml'))):
        urlopen(httpbin.url).read()

    # Callback should have made the file and the directory
    assert os.path.exists(str(tmpdir.join('nonexistent', 'cassette.yml.test')))


def test_load_cassette_with_custom_persister(tmpdir, httpbin):
    '''
    Ensure you can load a cassette using custom persister
    '''
    my_vcr = vcr.VCR()
    my_vcr.register_persister(CustomFilesystemPersister)

    test_fixture = str(tmpdir.join('synopsis.json.test'))

    with my_vcr.use_cassette(test_fixture, serializer='json'):
        response = urlopen(httpbin.url).read()
        assert b'difficult sometimes' in response



# flake8: noqa
import asyncio

import aiohttp
from aiohttp.test_utils import TestClient


async def aiohttp_request(loop, method, url, output='text', encoding='utf-8', content_type=None, **kwargs):
    session = aiohttp.ClientSession(loop=loop)
    response_ctx = session.request(method, url, **kwargs)

    response = await response_ctx.__aenter__()
    if output == 'text':
        content = await response.text()
    elif output == 'json':
        content_type = content_type or 'application/json'
        content = await response.json(encoding=encoding, content_type=content_type)
    elif output == 'raw':
        content = await response.read()

    response_ctx._resp.close()
    await session.close()

    return response, content


def aiohttp_app():
    async def hello(request):
        return aiohttp.web.Response(text='hello')

    app = aiohttp.web.Application()
    app.router.add_get('/', hello)
    return app



from vcr.stubs import VCRHTTPSConnection
from vcr.compat import mock
from vcr.cassette import Cassette


class TestVCRConnection(object):

    def test_setting_of_attributes_get_propogated_to_real_connection(self):
        vcr_connection = VCRHTTPSConnection('www.examplehost.com')
        vcr_connection.ssl_version = 'example_ssl_version'
        assert vcr_connection.real_connection.ssl_version == 'example_ssl_version'

    @mock.patch('vcr.cassette.Cassette.can_play_response_for', return_value=False)
    def testing_connect(*args):
        vcr_connection = VCRHTTPSConnection('www.google.com')
        vcr_connection.cassette = Cassette('test', record_mode='all')
        vcr_connection.real_connection.connect()
        assert vcr_connection.real_connection.sock is not None



#!/usr/bin/env python

import sys

from setuptools import setup, find_packages
from setuptools.command.test import test as TestCommand

long_description = open('README.rst', 'r').read()


class PyTest(TestCommand):

    def finalize_options(self):
        TestCommand.finalize_options(self)
        self.test_args = []
        self.test_suite = True

    def run_tests(self):
        # import here, cause outside the eggs aren't loaded
        import pytest
        errno = pytest.main(self.test_args)
        sys.exit(errno)


install_requires = [
    'PyYAML',
    'wrapt',
    'six>=1.5',
    'contextlib2; python_version=="2.7"',
    'mock; python_version=="2.7"',
    'yarl; python_version>="3.4"',
]

excluded_packages = ["tests*"]
if sys.version_info[0] == 2:
    excluded_packages.append("vcr.stubs.aiohttp_stubs")

setup(
    name='vcrpy',
    version='2.0.0',
    description=(
        "Automatically mock your HTTP interactions to simplify and "
        "speed up testing"
    ),
    long_description=long_description,
    author='Kevin McCarthy',
    author_email='me@kevinmccarthy.org',
    url='https://github.com/kevin1024/vcrpy',
    packages=find_packages(exclude=excluded_packages),
    python_requires='>=2.7, !=3.0.*, !=3.1.*, !=3.2.*, !=3.3.*',
    install_requires=install_requires,
    license='MIT',
    tests_require=['pytest', 'mock', 'pytest-httpbin'],
    classifiers=[
        'Development Status :: 4 - Beta',
        'Environment :: Console',
        'Intended Audience :: Developers',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: Implementation :: CPython',
        'Programming Language :: Python :: Implementation :: PyPy',
        'Topic :: Software Development :: Testing',
        'Topic :: Internet :: WWW/HTTP',
        'License :: OSI Approved :: MIT License',
    ]
)



# -*- coding: utf-8 -*-
'''Test using a proxy.'''

# External imports
import multiprocessing
import pytest

from six.moves import socketserver, SimpleHTTPServer
from six.moves.urllib.request import urlopen

# Internal imports
import vcr

# Conditional imports
requests = pytest.importorskip("requests")


class Proxy(SimpleHTTPServer.SimpleHTTPRequestHandler):
    '''
    Simple proxy server.

    (Inspired by: http://effbot.org/librarybook/simplehttpserver.htm).
    '''
    def do_GET(self):
        upstream_response = urlopen(self.path)
        try:
            status = upstream_response.status
            headers = upstream_response.headers.items()
        except AttributeError:
            # In Python 2 the response is an addinfourl instance.
            status = upstream_response.code
            headers = upstream_response.info().items()
        self.send_response(status, upstream_response.msg)
        for header in headers:
            self.send_header(*header)
        self.end_headers()
        self.copyfile(upstream_response, self.wfile)


@pytest.yield_fixture(scope='session')
def proxy_server():
    httpd = socketserver.ThreadingTCPServer(('', 0), Proxy)
    proxy_process = multiprocessing.Process(
        target=httpd.serve_forever,
    )
    proxy_process.start()
    yield 'http://{0}:{1}'.format(*httpd.server_address)
    proxy_process.terminate()


def test_use_proxy(tmpdir, httpbin, proxy_server):
    '''Ensure that it works with a proxy.'''
    with vcr.use_cassette(str(tmpdir.join('proxy.yaml'))):
        response = requests.get(httpbin.url, proxies={'http': proxy_server})

    with vcr.use_cassette(str(tmpdir.join('proxy.yaml'))) as cassette:
        cassette_response = requests.get(httpbin.url, proxies={'http': proxy_server})

    assert cassette_response.headers == response.headers
    assert cassette.play_count == 1
